function [channel_model, fitted_splines] = fit_global_parametric(pixels1, pixels2)
% fit a global parametric model based on constrained spline
% p1: pixels from im1
% p2: pixels from im2

%% Parameter section
color = {'r+', 'g+', 'b+'};
pixels1 = pixels1+1; % map from [0,255] -> [1,256]
pixels2 = pixels2+1;
fitted_splines = struct('x', [], 'y', []); % store fitted spline
channel_model = struct('model', []);

color_thres = 30;
diagonal_weight = 0.01;
debug = false;
ploton = 'off';
flag_gauss_weight = true; gauss_sigma = 20; gauss_mean = 0; gauss_offset = 30;

for i_channel = 1:3
    p1 = double(pixels1(:,i_channel));
    p2 = double(pixels2(:,i_channel));

    % Preprocessing of data
    %% Remove too abnormal color change, to make color transition more natural
    % Because we use least square fitting, it's sensitive to outliner
    % If change to more robust spline fit algorithm, this step may be
    % eliminated

%     outline_idx = (abs(p1-p2))>color_thres;
%     p1(outline_idx) = [];
%     p2(outline_idx) = [];
    

    %% Fit a spline to those data points
    % Fit a spline
    x = p1; y = p2;
    xy = [-25 -25; 275 275]; % constrain point for spline fitting

    % Try to reduce the effect of outliner match
    p1p2 = [x y];
    [u_p1p2, i_a, i_b] = unique(p1p2, 'rows');
    u_count = histc(i_b, 1:length(u_p1p2));
    count_mean = mean(u_count);
%     u_count(u_count > count_mean) = count_mean;
    u_count(u_count < count_mean) = 0;

    weight1 = u_count / max(u_count);
    
    x = u_p1p2(:,1); y = u_p1p2(:,2);
    
    weight2 = normpdf(max(abs(x-y)-gauss_offset, 0), gauss_mean, gauss_sigma); 
    % the smaller the sigma, the narrower the distribution
    % the stronger prior knowledge
    if flag_gauss_weight
        weight = weight1 .* weight2;
    else
        weight = weight1;
    end
    
    % visualize weight matrix
%     for i_pixel = 1:length(x)
%         correlation(256-y(i_pixel), x(i_pixel)) = weight(i_pixel);
%     end
%     imshow(correlation, []);
    % add diagonal constrain
%     x = [x; (1:256)']; y = [y; (1:256)'];
%     weight = [weight; ones(256, 1) * diagonal_weight];
    
    [model, xs, ys] = slmengine(x,y,'plot',ploton,'knots',(-25:25:275),'increasing','on', ... 
    'xy', xy, 'minslope', 0.1, 'weights', weight);
    fig_model = gcf;
    if strcmp(ploton, 'on'), title(color{i_channel}); end;

    fitted_splines(i_channel).x = xs; fitted_splines(i_channel).y = ys;
    channel_model(i_channel).model = model;
    
    % plot color correlation of each channel, debug
    if debug
        fig_correlation = plot_correlation(p1, p2);
        save_figure(fig_correlation, [color{i_channel} '_correlation.png']);
        save_figure(fig_model, [color{i_channel} '_fit.png']);
    end
    
    
    %{
    %% Fit the model inverse direction
    x = double(p2); y = double(p1);
    model = slmengine(x, y, 'plot', ploton, 'knots', (-25:50:275), 'increasing', 'on',...
    'xy', xy, 'minslope', 0.1);
    xp = 1:256;
    yp = slmeval(xp, model, 0);
    yp(yp<1)=1; yp(yp>256)=256;
    tuned_im2(:,:,channel) = yp(im2(:,:,channel)+1);
    tuned_im2(:,:,channel) = tuned_im2(:,:,channel)-1;
    %}
end % } end of rgb channel loop
    
end
