function pyramid = ssf_multiscale_dsift(raw_im, sigmas, pad_image)
	% sigmas
    % pad_image : whether enlarge original image to avoid empty border of
    % sift descriptor
    verbose = false;
    
    pyramid = struct('scale', [], 'sift', []);
    I = single(rgb2gray(raw_im));

    magnif = 3;
%     if ~exist('sigmas', 'var')
%         bin_sizes = [3 6 9 12];
%     else
    bin_sizes = sigmas .* magnif;
%     end
    
    if pad_image
        % enlarge the image to avoid the empty border of sift feature,
        % referenced from SLS paper
        %Calculate the maximal padding size according to vl_sift code for sift-descriptors
        NBP=4; %number of spatial bins while calculating sift-descriptor
        SBP=3*sigmas(end); %maximal extension of each spacial bin
        w=SBP*(NBP+1);
        pad_size=ceil(w/2); % padding size for maximal window size
        I = padarray(I, [pad_size pad_size], 'symmetric');
    end
    
    for i = 1:length(bin_sizes)
        bin_size = bin_sizes(i);
        
        scale = bin_size / magnif;

        Is = vl_imsmooth(I, sqrt(scale^2 - 0.25));
        [f, d] = vl_dsift(Is, 'size', bin_size);
        sift = zeros(size(Is, 1), size(Is, 2), 128, 'double');
        f = floor(f);
        for j = 1:size(d,2)
            sift(f(2,j), f(1,j), :) = d(:,j);
        end
        
        % normalize sift feature to length 1 of l2 norm
        [nrows, ncols, cols] = size(sift);
        % normalize SIFT descriptors
        sift_arr = reshape(sift, [nrows*ncols cols]);
        sift_arr = ssf_normalize_sift(sift_arr);
        sift = reshape(sift_arr, [nrows ncols cols]);
        
        if pad_image
            % crop the sift feature back to original size
            sift = sift(pad_size+1:end-pad_size,pad_size+1:end-pad_size,:);
            assert(size(sift,1)==size(raw_im,1) && size(sift,2)==size(raw_im, 2));
        end
        
        pyramid(i).scale = scale;
        pyramid(i).sift = sift;
        
        if verbose
            figure; imshow(showColorSIFT(sift));
        end
    end
end


